import { EventEmitter } from 'events';
import fs from 'fs';
import axios from 'axios';
import path from 'path';
import archiver from 'archiver';
import followRedirects from 'follow-redirects';
import qs from 'querystring';

var find = async function find(dir, axios) {
  try {
    var res = await axios({
      method: 'PROPFIND',
      url: path.join('/', dir),
      validateStatus: function validateStatus(status) {
        return status === 207 || status === 404;
      }
    });
    return res.status === 207;
  } catch (e) {
    throw new Error(e);
  }
};

var mkdir = async function mkdir(dir, axios) {
  try {
    var res = await axios({
      method: 'MKCOL',
      url: path.join('/', dir),
      validateStatus: function validateStatus(status) {
        return status < 500;
      }
    });
    return res.status === 201;
  } catch (e) {
    throw new Error(e);
  }
};

var zip = async function zip(src, dest, name) {
  return new Promise(function (resolve, reject) {
    var archive = archiver('zip', {
      zlib: {
        level: 9
      }
    });
    var file = path.resolve(dest, name + ".zip");
    var output = fs.createWriteStream(file);
    output.on('close', function () {
      resolve(file);
    });
    archive.on('error', function (error) {
      reject(error);
    });
    archive.pipe(output);
    src.forEach(function (sourcePath) {
      archive.glob('**/*', {
        cwd: sourcePath.source,
        ignore: sourcePath.exclude
      });
    });
    archive.finalize();
  });
};

followRedirects.maxBodyLength = 100 * 1024 * 1024;

var upload = async function upload(src, dest, axios) {
  try {
    var file = path.join('/', dest, path.basename(src));
    var stream = fs.createReadStream(src);
    var res = await axios({
      url: file,
      method: 'PUT',
      data: stream,
      maxRedirects: 0,
      validateStatus: function validateStatus(status) {
        return status < 500;
      }
    });
    return res.status === 201 ? file : false;
  } catch (e) {
    throw new Error(e);
  }
};

var unzip = async function unzip(file, axios) {
  try {
    var res = await axios({
      url: path.join('/', file),
      method: 'POST',
      data: qs.stringify({
        method: 'UNZIP'
      }),
      validateStatus: function validateStatus(status) {
        return status < 500;
      }
    });
    return res.status === 204;
  } catch (e) {
    throw new Error(e);
  }
};

var rm = async function rm(file, axios) {
  try {
    var res = await axios({
      method: 'DELETE',
      url: path.join('/', file),
      validateStatus: function validateStatus(status) {
        return status < 500;
      }
    });
    return res.status === 204;
  } catch (e) {
    throw new Error(e);
  }
};

var deploy = async function deploy(_ref) {
  var hostname = _ref.hostname,
      username = _ref.username,
      password = _ref.password,
      cartridges = _ref.cartridges,
      codeVersion = _ref.codeVersion,
      _ref$force = _ref.force,
      force = _ref$force === void 0 ? false : _ref$force,
      _ref$emitter = _ref.emitter,
      emitter = _ref$emitter === void 0 ? new EventEmitter() : _ref$emitter;

  if (!hostname) {
    throw new Error('Missing "Instance Hostname"');
  }

  if (!username) {
    throw new Error('Missing "Business Manager Username"');
  }

  if (!password) {
    throw new Error('Missing "Business Manager Password"');
  }

  if (!codeVersion) {
    throw new Error('Missing "Code Version"');
  }

  if (!cartridges) {
    throw new Error('Missing "Cartridges" source');
  }

  if (typeof cartridges === 'string') {
    cartridges = [{
      source: cartridges
    }];
  }

  try {
    cartridges.map(function (c) {
      return c.source;
    }).forEach(fs.accessSync);
  } catch (e) {
    throw new Error('Cartridges property contains invalid folders');
  }

  var instance = axios.create({
    baseURL: "https://" + hostname + "/on/demandware.servlet/webdav/Sites",
    auth: {
      username: username,
      password: password
    }
  });
  var dest = "/Cartridges/" + codeVersion;

  try {
    if (force) {
      emitter.emit('force', codeVersion);
      await rm(dest, instance);
    }

    emitter.emit('checkCodeVersion', codeVersion);
    var codeVersionExists = await find(dest, instance);

    if (codeVersionExists) {
      throw new Error('Code version exists');
    }

    emitter.emit('mkdir', dest);
    await mkdir(dest, instance);
    var file;
    emitter.emit('zip', cartridges);
    file = await zip(cartridges, process.env.TMPDIR || '.', codeVersion);
    emitter.emit('upload', file);
    file = await upload(file, dest, instance);
    emitter.emit('unzip', file);
    await unzip(file, instance);
    emitter.emit('rm', file);
    await rm(file, instance);
    emitter.emit('deployed', true);
    return file;
  } catch (e) {
    emitter.emit('deployed', false);
    throw new Error(e);
  }
};

export default deploy;
